#ifdef IPNAT

#if !defined(__SVR4) && !defined(__svr4__)
#include <strings.h>
#else
#include <sys/byteorder.h>
#endif
#include <sys/types.h>
#include <sys/time.h>
#include <sys/param.h>
#include <stdlib.h>
#include <unistd.h>
#include <stddef.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#if defined(sun) && (defined(__svr4__) || defined(__SVR4))
# include <sys/ioccom.h>
# include <sys/sysmacros.h>
#endif
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <net/if.h>
#include <netdb.h>
#include <arpa/nameser.h>
#include <arpa/inet.h>
#include <resolv.h>
#include <ctype.h>
#include "netinet/ip_compat.h"
#include "netinet/ip_fil.h"
#include "netinet/ip_nat.h"
#include "netinet/ip_state.h"
#include "netinet/ip_proxy.h"
#include "netinet/ip_nat.h"

int ipfilter_c_add_rdr(int fd, char *interface, char *addr_from, char *addr_dest, char *addr_rdr, int port_from, int port_dest, int port_rdr)
{
    int r;
    ipnat_t *nat;

    nat = malloc(sizeof(ipnat_t));

    memset(nat, 0x00, sizeof(ipnat_t));

    nat->in_redir = NAT_REDIRECT;

/* Source */
    nat->in_srcip = inet_addr(addr_from);
    if( nat->in_srcip == 0 ){
	nat->in_srcmsk = 0x0;    
    } else {
	nat->in_srcmsk = 0xffffffff;
    }

    if (port_from) {
	nat->in_sport = port_from;
	nat->in_scmp = FR_EQUAL;
    }

/* Dest */

    nat->in_outip = inet_addr(addr_dest);
    if( nat->in_outip==0 ){
        nat->in_outmsk = 0x0;
    } else {
	nat->in_outmsk = 0xffffffff;
    }

    if (port_dest) {
	nat->in_dport = port_dest;
	nat->in_dcmp = FR_EQUAL;
    }

/* RDR */
    nat->in_inip = inet_addr(addr_rdr);
    nat->in_inmsk = 0xffffffff;
    nat->in_pnext = ntohs(port_rdr);

    nat->in_space = ~(0xffffffff & ntohl(nat->in_outmsk));

    nat->in_flags = IPN_TCP | IPN_FILTER;

    strncpy(nat->in_ifname, interface, 10);

    r = ioctl(fd, SIOCADNAT, &nat);

    free( nat );

    return r;
}
int ipfilter_c_del_rdr(int fd, char *interface, char *addr_from, char *addr_dest, char *addr_rdr, int port_from, int port_dest, int port_rdr)
{
    int r;
    ipnat_t *nat;

    nat = malloc(sizeof(ipnat_t));

    memset(nat, 0x00, sizeof(ipnat_t));

    nat->in_redir = NAT_REDIRECT;

/* Source */
    nat->in_srcip = inet_addr(addr_from);
    if( nat->in_srcip == 0 ){
	nat->in_srcmsk = 0x0;    
    } else {
	nat->in_srcmsk = 0xffffffff;
    }

    if (port_from) {
	nat->in_sport = port_from;
	nat->in_scmp = FR_EQUAL;
    }

/* Dest */

    nat->in_outip = inet_addr(addr_dest);
    if( nat->in_outip==0 ){
        nat->in_outmsk = 0x0;
    } else {
	nat->in_outmsk = 0xffffffff;
    }

    if (port_dest) {
	nat->in_dport = port_dest;
	nat->in_dcmp = FR_EQUAL;
    }
//    nat->in_pmin   = htons( port_dest );
//    nat->in_pmax   = nat->in_pmin;

/* RDR */
    nat->in_inip = inet_addr(addr_rdr);
    nat->in_inmsk = 0xffffffff;
    nat->in_pnext = ntohs(port_rdr);

    nat->in_space = ~(0xffffffff & ntohl(nat->in_outmsk));

    nat->in_flags = IPN_TCP | IPN_FILTER;

    strncpy(nat->in_ifname, interface, 10);

    r = ioctl(fd, SIOCRMNAT, &nat);

    free( nat );

    return r;
}
int ipfilter_l_add_map(int fd, char *interface, unsigned long addr_from, unsigned long addr_dest, unsigned long addr_rdr, int port_from, int port_dest, int port_rdr)
{
    int r;
    ipnat_t *nat;

    nat = malloc(sizeof(ipnat_t));

    memset(nat, 0x00, sizeof(ipnat_t));

    nat->in_redir = NAT_MAP;

/* Source */
    nat->in_srcip = addr_dest;
    if (nat->in_srcip == 0) {
	nat->in_srcmsk = 0x0;
    } else {
	nat->in_srcmsk = 0xffffffff;
    }

    if (port_from) {
	nat->in_sport = port_from;
	nat->in_scmp = FR_EQUAL;
    }

    nat->in_outip = addr_rdr;
    if (nat->in_outip == 0) {
	nat->in_outmsk = 0x0;
    } else {
	nat->in_outmsk = 0xffffffff;
    }

    if (port_dest) {
	nat->in_dport = port_dest;
	nat->in_dcmp = FR_EQUAL;
    }

    nat->in_inip = addr_from;
    nat->in_inmsk = 0xffffffff;
    nat->in_pnext = ntohs(port_from);

    nat->in_space = ~(0xffffffff & ntohl(nat->in_outmsk));

    nat->in_flags = IPN_TCPUDP | IPN_FILTER;

    strncpy(nat->in_ifname, interface, 10);

    r = ioctl(fd, SIOCADNAT, &nat);

    free(nat);

    return r;
}
int ipfilter_l_del_map(int fd, char *interface, unsigned long addr_from, unsigned long addr_dest, unsigned long addr_rdr, int port_from, int port_dest, int port_rdr)
{
    int r;
    ipnat_t *nat;

    nat = malloc(sizeof(ipnat_t));

    memset(nat, 0x00, sizeof(ipnat_t));

    nat->in_redir = NAT_MAP;

    nat->in_srcip = addr_dest;
    if (nat->in_srcip == 0) {
	nat->in_srcmsk = 0x0;
    } else {
	nat->in_srcmsk = 0xffffffff;
    }

    if (port_from) {
	nat->in_sport = port_from;
	nat->in_scmp = FR_EQUAL;
    }

    nat->in_outip = addr_rdr;
    if (nat->in_outip == 0) {
	nat->in_outmsk = 0x0;
    } else {
	nat->in_outmsk = 0xffffffff;
    }

    if (port_dest) {
	nat->in_dport = port_dest;
	nat->in_dcmp = FR_EQUAL;
    }

/* RDR */
    nat->in_inip = addr_from;
    nat->in_inmsk = 0xffffffff;
    nat->in_pnext = ntohs(port_from);

    nat->in_space = ~(0xffffffff & ntohl(nat->in_outmsk));

    nat->in_flags = IPN_TCPUDP | IPN_FILTER;

    strncpy(nat->in_ifname, interface, 10);

    r = ioctl(fd, SIOCRMNAT, &nat);

    free(nat);

    return r;
}


int ipfilter_c_add_map(int fd, char *interface, char * addr_from, char * addr_dest, char * addr_rdr, int port_from, int port_dest, int port_rdr)
{
    int r;
    ipnat_t *nat;

    nat = malloc(sizeof(ipnat_t));

    memset(nat, 0x00, sizeof(ipnat_t));

    nat->in_redir = NAT_MAP;

/* Source */
    nat->in_srcip = inet_addr( addr_dest );
    if (nat->in_srcip == 0) {
	nat->in_srcmsk = 0x0;
    } else {
	nat->in_srcmsk = 0xffffffff;
    }

    if (port_from) {
	nat->in_sport = port_from;
	nat->in_scmp = FR_EQUAL;
    }

    nat->in_outip = inet_addr( addr_rdr );
    if (nat->in_outip == 0) {
	nat->in_outmsk = 0x0;
    } else {
	nat->in_outmsk = 0xffffffff;
    }

    if (port_dest) {
	nat->in_dport = port_dest ;
	nat->in_dcmp = FR_EQUAL;
    }

    nat->in_inip = inet_addr( addr_from );
    nat->in_inmsk = 0xffffffff;
    nat->in_pnext = ntohs(port_from);

    nat->in_space = ~(0xffffffff & ntohl(nat->in_outmsk));

    nat->in_flags = IPN_TCPUDP | IPN_FILTER;

    strncpy(nat->in_ifname, interface, 10);

    r = ioctl(fd, SIOCADNAT, &nat);

    free(nat);

    return r;
}
int ipfilter_c_del_map(int fd, char *interface, char * addr_from, char * addr_dest, char * addr_rdr, int port_from, int port_dest, int port_rdr)
{
    int r;
    ipnat_t *nat;

    nat = malloc(sizeof(ipnat_t));

    memset(nat, 0x00, sizeof(ipnat_t));

    nat->in_redir = NAT_MAP;

/* Source */
    nat->in_srcip = inet_addr( addr_dest );
    if (nat->in_srcip == 0) {
	nat->in_srcmsk = 0x0;
    } else {
	nat->in_srcmsk = 0xffffffff;
    }

    if (port_from) {
	nat->in_sport = port_from;
	nat->in_scmp = FR_EQUAL;
    }

    nat->in_outip = inet_addr( addr_rdr );
    if (nat->in_outip == 0) {
	nat->in_outmsk = 0x0;
    } else {
	nat->in_outmsk = 0xffffffff;
    }

    if (port_dest) {
	nat->in_dport = port_dest ;
	nat->in_dcmp = FR_EQUAL;
    }

    nat->in_inip = inet_addr( addr_from );
    nat->in_inmsk = 0xffffffff;
    nat->in_pnext = ntohs(port_from);

    nat->in_space = ~(0xffffffff & ntohl(nat->in_outmsk));

    nat->in_flags = IPN_TCPUDP | IPN_FILTER;

    strncpy(nat->in_ifname, interface, 10);

    r = ioctl(fd, SIOCRMNAT, &nat);

    free(nat);

    return r;
}

#endif
